@@ -1,10 +1,9 @@
 ###################################################
-### chunk number 1: 
+### chunk number 1:
 ###################################################
 
 compMatrix.writeTable <- function (compMatrix)
 {
-    xtable::xtable(compMatrix, display = c("s", rep("d", 4), rep("G", 4)), vsep = "|")
+    .Deprecated(package = "surveillance")
+    xtable(compMatrix, display = c("s", rep("d", 4), rep("G", 4)), vsep = "|")
 }
-
-
